<?php

namespace Common\Api;

class AlipayTransferApi extends BaseApi
{
    public $wg = 'https://openapi.alipaydev.com/gateway.do';
    public $sign_type = 'RSA2';
    public $private_key = null;
    public $app_id;
    public $charset = "UTF-8";
    public $format = "json";
    public $payee_type = "ALIPAY_LOGONID";
    public $payee_account = 'ohpbrq7462@sandbox.com';
    public $amount = 0;
    public $remark = '';
    public $pzid = 0;
    public $out_biz_no = 0;

    public function __construct()
    {
        $this->amount = $_REQUEST['amount'];
        $this->pzid   = $_REQUEST['pzid'];
        $this->out_biz_no   = $_REQUEST['out_biz_no'];
        $this->payee_account   = $_REQUEST['payee_account'];
    }

    //参数效验
    public function config_verify()
    {
        if($this->amount <=0 || $this->amount > 5000)     return [0, '单次提现金额0-5000'];
        if(empty($this->pzid)|| !is_numeric($this->pzid)) return [0, '转账支付宝账号不存在'];
        if(empty($this->out_biz_no)) return [0, '商户转账订单号不能为空'];
        if(empty($this->payee_account)) return [0, '收款账号不存在'];

        $data = SM('Jiekoupeizhi')->findData('*', 'pzid="' .$this->pzid. '"');

        if(empty($data))               return [0, '转账支付宝账号不存在'];
        if($data['style'] != 'alipay') return [0, '仅限企业支付宝转账申请'];

        $params = unserialize($data['params']);
        if(empty($params['alipay_appid']) || empty($params['alipay_private'])) return [0, '转账支付宝账号参数存在异常'];

        $payee = SM('Shoukuan')->findData('skid', 'zhanghao="' .$this->payee_account. '"');
        if(empty($payee))
            return [0, '收款账号不存在'];

        $this->private_key = $params['alipay_private'];
        $this->app_id      = $params['alipay_appid'];
        $this->sign_type   = $params['alipay_sign'];
        return true;
    }

    //发起转账申请
    public function transfer()
    {
        $result = $this->config_verify();
        if($result !== true)
            return $result;

        $biz = array(
            'out_biz_no' => $this->out_biz_no,
            'payee_type' => $this->payee_type,
            'payee_account' => $this->payee_account,
            'amount' => $this->amount,
            'remark' => $this->remark,
        );

        $data = array(
            'app_id' => $this->app_id,
            'method' => 'alipay.fund.trans.toaccount.transfer',
            'format' => $this->format,
            'charset'=> $this->charset,
            'sign_type' => $this->sign_type,
            'timestamp' => date('Y-m-d H:i:s'),
            'version'   => '1.0',
            'biz_content' => json_encode($biz, JSON_UNESCAPED_UNICODE),
        );

        $requestUrl = $this->build_request($data);

        $result = CURL($requestUrl);
        if(!empty($result)){
            $result = json_decode($result, true);
            $r = $result['alipay_fund_trans_toaccount_transfer_response'];
            if(!empty($r['code']) && $r['code'] == '10000'){
                return [1, array(
                    'order_id'=>$r['order_id'],
                    'out_biz_no'=>$r['out_biz_no'],
                    'pay_date'=>$r['pay_date'],
                )];
            }
        }
        return [0, !empty($r) ? $r: 'unknown mistake'];
    }

    //格式化签名
    protected function format_secret_key()
    {
        if(!strstr($this->private_key, '-----')){
            $this->private_key = "-----BEGIN RSA PRIVATE KEY-----\n" .
                wordwrap($this->private_key, 64, "\n", true) .
                "\n-----END RSA PRIVATE KEY-----";
        }
        return $this->private_key;
    }

    //拼接转账请求
    public function build_request($data)
    {
        $sign = base64_encode($this->alipay_sign($data));
        $data['sign'] = $sign;

        $requestUrl = $this->wg . "?";
        foreach ($data as $sysParamKey => $sysParamValue) {
            $requestUrl .= "$sysParamKey=" . urlencode($sysParamValue) . "&";
        }
        $requestUrl = substr($requestUrl, 0, -1);
        return $requestUrl;
    }

    //支付宝签名
    protected function alipay_sign($data)
    {
        ksort($data);
        $str = '';
        foreach($data as $i => $iData){
            $str .= $i . '=' . $iData . '&';
        }
        $str = substr($str, 0, -1);
        $key = $this->format_secret_key();
        if("RSA2" == $this->sign_type){
            openssl_sign($str, $sign, $key, OPENSSL_ALGO_SHA256);
        }else{
            openssl_sign($str, $sign, $key);
        }
        return $sign;
    }
}