<?php
namespace Common\Logic;

use Think\Log;

class AutoTransferLogic extends BaseLogic
{
    protected $notify = array();
    protected $api_url = 'http://%s/index/alipay/transfer.html';
    protected $log_status = array(0=>'未处理',1=>'处理成功',2=>'处理失败',3=>'超时等未知因素');
    const KEY = '261c090c5c3d7169b029ae62e6821d37';

    public function __construct() {
        parent::__construct();
        $this->api_url = sprintf($this->api_url, $_SERVER['SERVER_NAME'].($_SERVER['SERVER_PORT'] != 80 ? ':'.$_SERVER['SERVER_PORT'] : ''));
    }

    //记录转账日志
    public function record($data)
    {
        if(!empty($data['style']) && $data['is_allow_transfer']){
            switch ($data['style']){
                case 'alipay':
                    return $this->alipay($data);
                    break;
            }
        }
        return false;
    }

    //转账处理
    public function transfer()
    {
        $data = $this->get_pending_transfer_log();
        if(!empty($data))
        {
            $receipt_account = $this->get_valid_payee_account();
            if(empty($receipt_account))
                return false;

            $data['payee_account'] = $receipt_account['zhanghao'];
            $data['ts'] = time();
            $data['sign'] = md5(self::KEY.$data['pzid'].$data['amount'].$data['ts'].$data['out_biz_no']);
            $update['update_time'] = time();
            $update['payee_account'] = $receipt_account['zhanghao'];

            $r = CURL($this->api_url, $data);
            if(!empty($r))
            {
                $r = json_decode($r, true);
                if($r[0] == '1'){
                    $update['trade_no'] = $r[1]['order_id'];
                    $update['pay_time'] = strtotime($r[1]['pay_date']);
                    $update['status']   = 1;
                }else{
                    $update['status'] = 2;
                    $update['extra']  = serialize($r);
                    $this->close_receipt_account($data['pzid'], 0,'receipt');
                    $this->close_receipt_account($receipt_account['skid'], 0,'transfer');
                    $this->notify(2);
                }
            }else{
                $update['status'] = 3;
            }

            $res = $this->update_pending_transfer_log($data['log_id'], $update);
            if($update['status'] == 1 && $res){
                $this->change_amount($receipt_account['skid'], $data['amount']);
            }
            return '['.$this->format_log_status($update['status']).'] [LOG_ID:'.$data['log_id'].'] ['.$res.']';
        }
        return false;
    }

    //支付宝转账记录
    public function alipay($data)
    {
        if($data['is_allow_transfer'] == 0)
            return true;

        return SM('Alipay_transfer_log')->insertData(
            array(
                'out_biz_no'=>$data['out_biz_no'],
                'pzid'=>$data['pzid'],
                'amount'=>$data['amount'],
                'create_time'=>time(),
            )
        );
    }

    //获取待处理记录
    protected function get_pending_transfer_log()
    {
        return SM('Alipay_transfer_log')->findData('log_id,pzid,amount,out_biz_no,log_id', 'status = 0' , 'priority DESC,log_id ASC');
    }

    //更新待处理记录状态
    protected function update_pending_transfer_log($id, $data)
    {
        if(empty($id) || empty($data))
            return false;
        return SM('Alipay_transfer_log')->UpdateData($data, 'log_id='.$id);
    }

    //格式化处理状态
    protected function format_log_status($status)
    {
        return isset($this->log_status[$status]) ? $this->log_status[$status] : $this->log_status[3];
    }

    //关闭开启收款、转账账号
    protected function close_receipt_account($id = 0, $status = 0, $type = 'receipt')
    {
        if(empty($id) &&
            in_array($status, [0,1]) &&
            in_array($type, ['receipt','transfer']))
            return false;
        if($type == 'receipt')  return SM('Jiekouzj')->updateData(array('ifopen'=>$status,'ifchoose'=>$status), "pzid='{$id}'");
        if($type == 'transfer') return SM('Shoukuan')->updateData(array('status'=>$status), "skid='{$id}'");
        return false;
    }

    //获取收款账号
    protected function get_payee_account()
    {
        return SM('Shoukuan')->selectData('skid,zhanghao,username,day_money,money,dayxian,shangxian', "status = 1");
    }

    //效验收款账号合法性
    protected function get_valid_payee_account()
    {
        $accounts = $this->get_payee_account();
        if(!empty($accounts))
        {
            $normal = array();
            $fail = array();
            foreach($accounts as $account)
            {
                if((($account['dayxian'] > 0) && ($account['day_money'] >= $account['dayxian'])) ||
                   (($account['shangxian'] > 0) && ($account['money'] >= $account['shangxian']))){
                    $fail[] = $account['skid'];
                    continue;
                }
                $normal[] = $account;
            }
            if(!empty($fail)) SM('Shoukuan')->updateData(array('status'=>0), "skid in(".implode(',', $fail).")");
            if(!empty($normal)) return $normal[rand(0, count($normal)-1)];
        }
        $this->notify(1);
        return false;
    }

    //更改收款账号金额
    protected function change_amount($id = 0, $money = 0)
    {
        if(empty($id) || empty($money))
            return  false;
        return M()->execute("UPDATE fx_shoukuan SET day_money=day_money+{$money},money=money+{$money} where skid='{$id}'");
    }

    //运维消息提醒  临时假装通知到运维人员 fuck
    protected function notify($type = 0){
        switch($type){
            case '1':
                Log::record('无可用转账收款账号');
                break;
            case '2':
                Log::record('支付通道已关闭');
                break;
        }
        return true;
    }
}
