<?php

namespace Common\Logic;

class NewsLogic extends BaseLogic {

    var $moduleName = '站内新闻';

    /**
     * 浏览动态列表
     * @author fengxing
     */
    public function index($request) {
        $news = SM('News');
        $map = array();
        $data = ' 1=1 ';
        //高级查询
        if ($request['title']) {
            $map['title'] = $request['title'];
            $data .= ' AND title like "%' . $request['title'] . '%" ';
        }
        if (is_numeric($request['status'])) {
            $map['status'] = $request['status'];
            $data .= ' AND status = "' . $request['status'] . '" ';
        }
        $perpage = C('FX_PERPAGE');
        $count = $news->selectCount(
                $data, "*"); // 查询满足要求的总记录数
        // 进行分页数据查询 注意limit方法的参数要使用Page类的属性
        $page = page($count, $request['p'], $perpage) . ',' . $perpage;
        $list = $news->pageData(
                '*', $data, 'id DESC', $page);

        foreach ($list as $i => $iList) {
            $list[$i]['addtime'] = stringChange('formatDateTime', $iList['addtime']);
            $list[$i]['edittime'] = stringChange('formatDateTime', $iList['edittime']);
            $list[$i]['status'] = $list[$i]['status'] == 1 ? '锁定' : '正常';
        }

        $pageList = $this->pageList($count, $perpage, $map);

        $params = array(
            'list' => $list,
            'page' => $pageList,
            'pageName' => $this->moduleName . '管理'
        );
        return [
            1,
            $params];
    }

    /**
     * 添加动态
     * @author fengxing
     */
    public function add() {

        $params = array(
            'act' => 'add',
            'pageName' => '添加' . $this->moduleName
        );
        return [
            1,
            $params];
    }

    /**
     * 编辑动态
     * @author fengxing
     */
    public function edit($request) {
        $newID = (int) $request['id']; //获取数据标识
        //判断数据标识
        if (!$newID) {
            return [
                0,
                '参数错误！'];
        }
        $edit = SM('News')->findData('*', 'id="' . $newID . '"');
        $edit['colorInt'] = str_replace('#', '', $edit['color']);
        $edit['content'] = stripslashes($edit['content']);
        $jsonEdit = json_encode($edit);

        $params = array(
            'data' => $jsonEdit,
            'edit' => $edit,
            'act' => 'edit',
            'pageName' => '修改' . $this->moduleName
        );
        return [
            1,
            $params,
            'News/add'];
    }

    /**
     * 保存动态
     * @author fengxing
     */
    public function save($request) {
        $newID = $request['id']; //获取数据标识
        $act = $request['act']; //获取模板标识
        //判断数据标识
        if (empty($newID) && $act == 'edit') {
            return [
                0,
                '数据标识不能为空！'];
        }
        if (empty($act)) {
            return [
                0,
                '模板标识不能为空！'];
        }
        $news = SM('News');
        $data = array();
        $data['title'] = $request['title'];
        $data['content'] = $request['content'];
        $data['adminname'] = $this->getCookieUserName();
        $data['color'] = $request['color'];
        $data['status'] = $request['status'];

        if (empty($data['title'])) {
            return [
                0,
                '标题不能为空！'];
        }

        if ($act == 'add') {
            $data['addtime'] = time();
            //检查动态名称重复
            $buffer = $news->selectData(
                    '*', 'title="' . $data['title'] . '"');
            if ($buffer) {
                return [
                    0,
                    '名称重复请更换！'];
            }
            if ($news->insertData(
                            $data) === false) {
                return [
                    0,
                    '添加失败！'];
            } else {
                //写入日志
                $this->adminLog($this->moduleName, '添加动态【' . $request['title'] . '】');
                return [
                    1,
                    '添加成功！',
                    __URL__];
            }
        } else if ($act == 'edit') {
            $data['edittime'] = time();
            $data['id'] = $request['id'];
            if ($news->updateData(
                            $data, 'id = "' . $newID . '"') === false) {
                return [
                    0,
                    '修改失败！'];
            } else {
                //写入日志
                $this->adminLog($this->moduleName, '修改动态id为【' . $request['id'] . '】的数据');
                return [
                    1,
                    '修改成功！',
                    __URL__];
            }
        }
    }

    /**
     * 删除动态
     * @author fengxing
     */
    public function delete() {
        $newID = $request['id']; //获取数据标识
        if (!$newID) {
            return [
                0,
                '数据标识不能为空！'];
        }
        if (!is_array($newID))
            $newID = explode(',', $newID);

        $newID = implode('","', $newID);
        if (SM('News')->deleteData(
                        'id in ("' . $newID . '")') === false) {
            return [
                0,
                '删除失败！'];
        } else {
            //写入日志
            $this->adminLog($this->moduleName, '删除动态id为【"' . $newID . '"】的数据');
            return [
                1,
                '删除成功！',
                __URL__];
        }
    }

}
